@@ -1,4 +1,4 @@
-/* $PostgreSQL: pgsql/src/interfaces/ecpg/ecpglib/execute.c,v 1.38.4.1 2005/03/18 10:01:14 meskes Exp $ */
+/* $PostgreSQL: pgsql/src/interfaces/ecpg/ecpglib/execute.c,v 1.38.4.10 2006/08/18 16:33:50 meskes Exp $ */
 
 /*
  * The aim is to get a simpler inteface to the database routines.
@@ -66,10 +66,11 @@ quote_postgres(char *arg, int lineno)
 	res[ri++] = '\'';
 	res[ri] = '\0';
 
+	ECPGfree(arg);
 	return res;
 }
 
-#if defined(__GNUC__) && (defined (__powerpc__) || defined(__AMD64__))
+#if defined(__GNUC__) && (defined (__powerpc__) || defined(__AMD64__) || defined(__x86_64__))
 #define APREF ap
 #else
 #define APREF *ap
@@ -147,7 +148,7 @@ ECPGget_variable(va_list APREF, enum ECPGttype type, struct variable * var, bool
  * ind_offset - indicator offset
  */
 static bool
-create_statement(int lineno, int compat, int force_indicator, struct connection * connection, struct statement ** stmt, char *query, va_list ap)
+create_statement(int lineno, int compat, int force_indicator, struct connection * connection, struct statement ** stmt, const char *query, va_list ap)
 {
 	struct variable **list = &((*stmt)->inlist);
 	enum ECPGttype type;
@@ -155,7 +156,7 @@ create_statement(int lineno, int compat, int force_indicator, struct connection
 	if (!(*stmt = (struct statement *) ECPGalloc(sizeof(struct statement), lineno)))
 		return false;
 
-	(*stmt)->command = query;
+	(*stmt)->command = ECPGstrdup(query, lineno);
 	(*stmt)->connection = connection;
 	(*stmt)->lineno = lineno;
 	(*stmt)->compat = compat;
@@ -177,7 +178,7 @@ create_statement(int lineno, int compat, int force_indicator, struct connection
 			if (!(var = (struct variable *) ECPGalloc(sizeof(struct variable), lineno)))
 				return false;
 
-#if defined(__GNUC__) && (defined (__powerpc__) || defined(__AMD64__))
+#if defined(__GNUC__) && (defined (__powerpc__) || defined(__AMD64__) || defined(__x86_64__))
 			ECPGget_variable(ap, type, var, true);
 #else
 			ECPGget_variable(&ap, type, var, true);
@@ -230,6 +231,7 @@ free_statement(struct statement * stmt)
 		return;
 	free_variable(stmt->inlist);
 	free_variable(stmt->outlist);
+	ECPGfree(stmt->command);
 	ECPGfree(stmt);
 }
 
@@ -250,16 +252,20 @@ next_insert(char *text)
 	return (*ptr == '\0') ? NULL : ptr;
 }
 
-static void
+static bool
 ECPGtypeinfocache_push(struct ECPGtype_information_cache ** cache, int oid, bool isarray, int lineno)
 {
 	struct ECPGtype_information_cache *new_entry
 	= (struct ECPGtype_information_cache *) ECPGalloc(sizeof(struct ECPGtype_information_cache), lineno);
 
+	if (new_entry == NULL)
+		return (false);
+
 	new_entry->oid = oid;
 	new_entry->isarray = isarray;
 	new_entry->next = *cache;
 	*cache = new_entry;
+	return(true);
 }
 
 static enum ARRAY_TYPE
@@ -280,48 +286,48 @@ ECPGis_type_an_array(int type, const struct statement * stmt, const struct varia
 #define not_an_array_in_ecpg ECPG_ARRAY_NONE
 
 		/* populate cache with well known types to speed things up */
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), BOOLOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), BYTEAOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), CHAROID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), NAMEOID, not_an_array_in_ecpg, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), INT8OID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), INT2OID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), INT2VECTOROID, ECPG_ARRAY_VECTOR, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), INT4OID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), REGPROCOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), TEXTOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), OIDOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), TIDOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), XIDOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), CIDOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), OIDVECTOROID, ECPG_ARRAY_VECTOR, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), POINTOID, ECPG_ARRAY_VECTOR, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), LSEGOID, ECPG_ARRAY_VECTOR, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), PATHOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), BOXOID, ECPG_ARRAY_VECTOR, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), POLYGONOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), LINEOID, ECPG_ARRAY_VECTOR, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), FLOAT4OID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), FLOAT8OID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), ABSTIMEOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), RELTIMEOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), TINTERVALOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), UNKNOWNOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), CIRCLEOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), CASHOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), INETOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), CIDROID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), BPCHAROID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), VARCHAROID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), DATEOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), TIMEOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), TIMESTAMPOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), TIMESTAMPTZOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), INTERVALOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), TIMETZOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), ZPBITOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), VARBITOID, ECPG_ARRAY_NONE, stmt->lineno);
-		ECPGtypeinfocache_push(&(stmt->connection->cache_head), NUMERICOID, ECPG_ARRAY_NONE, stmt->lineno);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), BOOLOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), BYTEAOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), CHAROID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), NAMEOID, not_an_array_in_ecpg, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), INT8OID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), INT2OID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), INT2VECTOROID, ECPG_ARRAY_VECTOR, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), INT4OID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), REGPROCOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), TEXTOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), OIDOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), TIDOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), XIDOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), CIDOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), OIDVECTOROID, ECPG_ARRAY_VECTOR, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), POINTOID, ECPG_ARRAY_VECTOR, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), LSEGOID, ECPG_ARRAY_VECTOR, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), PATHOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), BOXOID, ECPG_ARRAY_VECTOR, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), POLYGONOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), LINEOID, ECPG_ARRAY_VECTOR, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), FLOAT4OID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), FLOAT8OID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), ABSTIMEOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), RELTIMEOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), TINTERVALOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), UNKNOWNOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), CIRCLEOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), CASHOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), INETOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), CIDROID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), BPCHAROID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), VARCHAROID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), DATEOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), TIMEOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), TIMESTAMPOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), TIMESTAMPTZOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), INTERVALOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), TIMETZOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), ZPBITOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), VARBITOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
+		if (!ECPGtypeinfocache_push(&(stmt->connection->cache_head), NUMERICOID, ECPG_ARRAY_NONE, stmt->lineno)) return (ECPG_ARRAY_ERROR);
 	}
 
 	for (cache_entry = (stmt->connection->cache_head); cache_entry != NULL; cache_entry = cache_entry->next)
@@ -331,6 +337,9 @@ ECPGis_type_an_array(int type, const struct statement * stmt, const struct varia
 	}
 
 	array_query = (char *) ECPGalloc(strlen("select typlen from pg_type where oid= and typelem<>0") + 11, stmt->lineno);
+	if (array_query == NULL)
+		return (ECPG_ARRAY_ERROR);
+
 	sprintf(array_query, "select typlen from pg_type where oid=%d and typelem<>0", type);
 	query = PQexec(stmt->connection->connection, array_query);
 	ECPGfree(array_query);
@@ -367,7 +376,11 @@ ECPGstore_result(const PGresult *results, int act_field,
 				ntuples = PQntuples(results);
 	bool		status = true;
 
-	isarray = ECPGis_type_an_array(PQftype(results, act_field), stmt, var);
+	if ((isarray = ECPGis_type_an_array(PQftype(results, act_field), stmt, var)) == ECPG_ARRAY_ERROR)
+	{
+		ECPGraise(stmt->lineno, ECPG_OUT_OF_MEMORY, ECPG_SQLSTATE_ECPG_OUT_OF_MEMORY, NULL);
+		return false;
+	}
 
 	if (isarray == ECPG_ARRAY_NONE)
 	{
@@ -439,6 +452,8 @@ ECPGstore_result(const PGresult *results, int act_field,
 				break;
 		}
 		var->value = (char *) ECPGalloc(len, stmt->lineno);
+		if (!var->value)
+			return false;
 		*((char **) var->pointer) = var->value;
 		ECPGadd_mem(var->value, stmt->lineno);
 	}
@@ -449,6 +464,8 @@ ECPGstore_result(const PGresult *results, int act_field,
 		int			len = var->ind_offset * ntuples;
 
 		var->ind_value = (char *) ECPGalloc(len, stmt->lineno);
+		if (!var->ind_value)
+			return false;
 		*((char **) var->ind_pointer) = var->ind_value;
 		ECPGadd_mem(var->ind_value, stmt->lineno);
 	}
@@ -561,19 +578,21 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 	}
 	if (**tobeinserted_p == '\0')
 	{
+		int asize = var->arrsize? var->arrsize : 1;
+
 		switch (var->type)
 		{
 				int			element;
 
 			case ECPGt_short:
-				if (!(mallocedval = ECPGalloc(var->arrsize * 20, lineno)))
+				if (!(mallocedval = ECPGalloc(asize * 20, lineno)))
 					return false;
 
-				if (var->arrsize > 1)
+				if (asize > 1)
 				{
 					strcpy(mallocedval, "array [");
 
-					for (element = 0; element < var->arrsize; element++)
+					for (element = 0; element < asize; element++)
 						sprintf(mallocedval + strlen(mallocedval), "%hd,", ((short *) var->value)[element]);
 
 					strcpy(mallocedval + strlen(mallocedval) - 1, "]");
@@ -586,14 +605,14 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 				break;
 
 			case ECPGt_int:
-				if (!(mallocedval = ECPGalloc(var->arrsize * 20, lineno)))
+				if (!(mallocedval = ECPGalloc(asize * 20, lineno)))
 					return false;
 
-				if (var->arrsize > 1)
+				if (asize > 1)
 				{
 					strcpy(mallocedval, "array [");
 
-					for (element = 0; element < var->arrsize; element++)
+					for (element = 0; element < asize; element++)
 						sprintf(mallocedval + strlen(mallocedval), "%d,", ((int *) var->value)[element]);
 
 					strcpy(mallocedval + strlen(mallocedval) - 1, "]");
@@ -606,14 +625,14 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 				break;
 
 			case ECPGt_unsigned_short:
-				if (!(mallocedval = ECPGalloc(var->arrsize * 20, lineno)))
+				if (!(mallocedval = ECPGalloc(asize * 20, lineno)))
 					return false;
 
-				if (var->arrsize > 1)
+				if (asize > 1)
 				{
 					strcpy(mallocedval, "array [");
 
-					for (element = 0; element < var->arrsize; element++)
+					for (element = 0; element < asize; element++)
 						sprintf(mallocedval + strlen(mallocedval), "%hu,", ((unsigned short *) var->value)[element]);
 
 					strcpy(mallocedval + strlen(mallocedval) - 1, "]");
@@ -626,14 +645,14 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 				break;
 
 			case ECPGt_unsigned_int:
-				if (!(mallocedval = ECPGalloc(var->arrsize * 20, lineno)))
+				if (!(mallocedval = ECPGalloc(asize * 20, lineno)))
 					return false;
 
-				if (var->arrsize > 1)
+				if (asize > 1)
 				{
 					strcpy(mallocedval, "array [");
 
-					for (element = 0; element < var->arrsize; element++)
+					for (element = 0; element < asize; element++)
 						sprintf(mallocedval + strlen(mallocedval), "%u,", ((unsigned int *) var->value)[element]);
 
 					strcpy(mallocedval + strlen(mallocedval) - 1, "]");
@@ -646,14 +665,14 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 				break;
 
 			case ECPGt_long:
-				if (!(mallocedval = ECPGalloc(var->arrsize * 20, lineno)))
+				if (!(mallocedval = ECPGalloc(asize * 20, lineno)))
 					return false;
 
-				if (var->arrsize > 1)
+				if (asize > 1)
 				{
 					strcpy(mallocedval, "array [");
 
-					for (element = 0; element < var->arrsize; element++)
+					for (element = 0; element < asize; element++)
 						sprintf(mallocedval + strlen(mallocedval), "%ld,", ((long *) var->value)[element]);
 
 					strcpy(mallocedval + strlen(mallocedval) - 1, "]");
@@ -666,14 +685,14 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 				break;
 
 			case ECPGt_unsigned_long:
-				if (!(mallocedval = ECPGalloc(var->arrsize * 20, lineno)))
+				if (!(mallocedval = ECPGalloc(asize * 20, lineno)))
 					return false;
 
-				if (var->arrsize > 1)
+				if (asize > 1)
 				{
 					strcpy(mallocedval, "array [");
 
-					for (element = 0; element < var->arrsize; element++)
+					for (element = 0; element < asize; element++)
 						sprintf(mallocedval + strlen(mallocedval), "%lu,", ((unsigned long *) var->value)[element]);
 
 					strcpy(mallocedval + strlen(mallocedval) - 1, "]");
@@ -686,14 +705,14 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 				break;
 #ifdef HAVE_LONG_LONG_INT_64
 			case ECPGt_long_long:
-				if (!(mallocedval = ECPGalloc(var->arrsize * 30, lineno)))
+				if (!(mallocedval = ECPGalloc(asize * 30, lineno)))
 					return false;
 
-				if (var->arrsize > 1)
+				if (asize > 1)
 				{
 					strcpy(mallocedval, "array [");
 
-					for (element = 0; element < var->arrsize; element++)
+					for (element = 0; element < asize; element++)
 						sprintf(mallocedval + strlen(mallocedval), "%lld,", ((long long *) var->value)[element]);
 
 					strcpy(mallocedval + strlen(mallocedval) - 1, "]");
@@ -706,14 +725,14 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 				break;
 
 			case ECPGt_unsigned_long_long:
-				if (!(mallocedval = ECPGalloc(var->arrsize * 30, lineno)))
+				if (!(mallocedval = ECPGalloc(asize * 30, lineno)))
 					return false;
 
-				if (var->arrsize > 1)
+				if (asize > 1)
 				{
 					strcpy(mallocedval, "array [");
 
-					for (element = 0; element < var->arrsize; element++)
+					for (element = 0; element < asize; element++)
 						sprintf(mallocedval + strlen(mallocedval), "%llu,", ((unsigned long long *) var->value)[element]);
 
 					strcpy(mallocedval + strlen(mallocedval) - 1, "]");
@@ -726,14 +745,14 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 				break;
 #endif   /* HAVE_LONG_LONG_INT_64 */
 			case ECPGt_float:
-				if (!(mallocedval = ECPGalloc(var->arrsize * 25, lineno)))
+				if (!(mallocedval = ECPGalloc(asize * 25, lineno)))
 					return false;
 
-				if (var->arrsize > 1)
+				if (asize > 1)
 				{
 					strcpy(mallocedval, "array [");
 
-					for (element = 0; element < var->arrsize; element++)
+					for (element = 0; element < asize; element++)
 						sprintf(mallocedval + strlen(mallocedval), "%.14g,", ((float *) var->value)[element]);
 
 					strcpy(mallocedval + strlen(mallocedval) - 1, "]");
@@ -746,14 +765,14 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 				break;
 
 			case ECPGt_double:
-				if (!(mallocedval = ECPGalloc(var->arrsize * 25, lineno)))
+				if (!(mallocedval = ECPGalloc(asize * 25, lineno)))
 					return false;
 
-				if (var->arrsize > 1)
+				if (asize > 1)
 				{
 					strcpy(mallocedval, "array [");
 
-					for (element = 0; element < var->arrsize; element++)
+					for (element = 0; element < asize; element++)
 						sprintf(mallocedval + strlen(mallocedval), "%.14g,", ((double *) var->value)[element]);
 
 					strcpy(mallocedval + strlen(mallocedval) - 1, "]");
@@ -819,8 +838,6 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 					if (!mallocedval)
 						return false;
 
-					ECPGfree(newcopy);
-
 					*tobeinserted_p = mallocedval;
 					*malloced_p = true;
 				}
@@ -855,8 +872,6 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 					if (!mallocedval)
 						return false;
 
-					ECPGfree(newcopy);
-
 					*tobeinserted_p = mallocedval;
 					*malloced_p = true;
 				}
@@ -866,55 +881,69 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 			case ECPGt_numeric:
 				{
 					char	   *str = NULL;
-					int			slen;
-					numeric    *nval = PGTYPESnumeric_new();
+					int	    slen;
+					numeric    *nval;
 
 					if (var->arrsize > 1)
 					{
 						for (element = 0; element < var->arrsize; element++)
 						{
+							nval = PGTYPESnumeric_new();
+							if (!nval)
+								return false;
+								
 							if (var->type == ECPGt_numeric)
 								PGTYPESnumeric_copy((numeric *) ((var + var->offset * element)->value), nval);
 							else
 								PGTYPESnumeric_from_decimal((decimal *) ((var + var->offset * element)->value), nval);
 
 							str = PGTYPESnumeric_to_asc(nval, nval->dscale);
-							PGTYPESnumeric_free(nval);
 							slen = strlen(str);
+							PGTYPESnumeric_free(nval);
 
 							if (!(mallocedval = ECPGrealloc(mallocedval, strlen(mallocedval) + slen + sizeof("array [] "), lineno)))
+							{
+								ECPGfree(str);
 								return false;
+							}
 
 							if (!element)
 								strcpy(mallocedval, "array [");
 
 							strncpy(mallocedval + strlen(mallocedval), str, slen + 1);
 							strcpy(mallocedval + strlen(mallocedval), ",");
+							ECPGfree(str);
 						}
 						strcpy(mallocedval + strlen(mallocedval) - 1, "]");
 					}
 					else
 					{
+						nval = PGTYPESnumeric_new();
+						if (!nval)
+							return false;
+
 						if (var->type == ECPGt_numeric)
 							PGTYPESnumeric_copy((numeric *) (var->value), nval);
 						else
 							PGTYPESnumeric_from_decimal((decimal *) (var->value), nval);
 
 						str = PGTYPESnumeric_to_asc(nval, nval->dscale);
-
-						PGTYPESnumeric_free(nval);
 						slen = strlen(str);
+						PGTYPESnumeric_free(nval);
 
 						if (!(mallocedval = ECPGalloc(slen + 1, lineno)))
+						{
+							free(str);
 							return false;
+						}
 
 						strncpy(mallocedval, str, slen);
 						mallocedval[slen] = '\0';
+						ECPGfree(str);
 					}
 
 					*tobeinserted_p = mallocedval;
 					*malloced_p = true;
-					free(str);
 				}
 				break;
 
@@ -928,10 +957,15 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 						for (element = 0; element < var->arrsize; element++)
 						{
 							str = quote_postgres(PGTYPESinterval_to_asc((interval *) ((var + var->offset * element)->value)), lineno);
+							if (!str)
+								return false;
 							slen = strlen(str);
 
 							if (!(mallocedval = ECPGrealloc(mallocedval, strlen(mallocedval) + slen + sizeof("array [],interval "), lineno)))
+							{
+								ECPGfree(str);
 								return false;
+							}
 
 							if (!element)
 								strcpy(mallocedval, "array [");
@@ -939,25 +973,31 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 							strcpy(mallocedval + strlen(mallocedval), "interval ");
 							strncpy(mallocedval + strlen(mallocedval), str, slen + 1);
 							strcpy(mallocedval + strlen(mallocedval), ",");
+							ECPGfree(str);
 						}
 						strcpy(mallocedval + strlen(mallocedval) - 1, "]");
 					}
 					else
 					{
 						str = quote_postgres(PGTYPESinterval_to_asc((interval *) (var->value)), lineno);
+						if (!str)
+							return false;
 						slen = strlen(str);
 
 						if (!(mallocedval = ECPGalloc(slen + sizeof("interval ") + 1, lineno)))
+						{
+							ECPGfree(str);
 							return false;
+						}
 
 						strcpy(mallocedval, "interval ");
 						/* also copy trailing '\0' */
 						strncpy(mallocedval + strlen(mallocedval), str, slen + 1);
+						ECPGfree(str);
 					}
 
 					*tobeinserted_p = mallocedval;
 					*malloced_p = true;
-					free(str);
 				}
 				break;
 
@@ -971,10 +1011,15 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 						for (element = 0; element < var->arrsize; element++)
 						{
 							str = quote_postgres(PGTYPESdate_to_asc(*(date *) ((var + var->offset * element)->value)), lineno);
+							if (!str)
+								return false;
 							slen = strlen(str);
 
 							if (!(mallocedval = ECPGrealloc(mallocedval, strlen(mallocedval) + slen + sizeof("array [],date "), lineno)))
+							{
+								ECPGfree(str);
 								return false;
+							}
 
 							if (!element)
 								strcpy(mallocedval, "array [");
@@ -988,10 +1033,15 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 					else
 					{
 						str = quote_postgres(PGTYPESdate_to_asc(*(date *) (var->value)), lineno);
+						if (!str)
+							return false;
 						slen = strlen(str);
 
 						if (!(mallocedval = ECPGalloc(slen + sizeof("date ") + 1, lineno)))
+						{
+							ECPGfree(str);
 							return false;
+						}
 
 						strcpy(mallocedval, "date ");
 						/* also copy trailing '\0' */
@@ -1000,24 +1050,35 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 
 					*tobeinserted_p = mallocedval;
 					*malloced_p = true;
-					free(str);
+					ECPGfree(str);
 				}
 				break;
 
 			case ECPGt_timestamp:
 				{
-					char	   *str = NULL;
+					char	   *str = NULL, *asc = NULL;
 					int			slen;
 
 					if (var->arrsize > 1)
 					{
 						for (element = 0; element < var->arrsize; element++)
 						{
-							str = quote_postgres(PGTYPEStimestamp_to_asc(*(timestamp *) ((var + var->offset * element)->value)), lineno);
+							asc = PGTYPEStimestamp_to_asc(*(timestamp *) ((var + var->offset * element)->value));
+							if (!asc)
+								return false;
+
+							str = quote_postgres(asc, lineno);
+							ECPGfree(asc); /* we don't need this anymore so free it asap. */
+							if (!str)
+								return false;
+
 							slen = strlen(str);
 
 							if (!(mallocedval = ECPGrealloc(mallocedval, strlen(mallocedval) + slen + sizeof("array [], timestamp "), lineno)))
+							{
+								ECPGfree(str);
 								return false;
+							}
 
 							if (!element)
 								strcpy(mallocedval, "array [");
@@ -1030,11 +1091,21 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 					}
 					else
 					{
-						str = quote_postgres(PGTYPEStimestamp_to_asc(*(timestamp *) (var->value)), lineno);
+						asc = PGTYPEStimestamp_to_asc(*(timestamp *) (var->value));
+						if (!asc)
+							return false;
+
+						str = quote_postgres(asc, lineno);
+						ECPGfree(asc); /* we don't need this anymore so free it asap. */
+						if (!str)
+							return false;
 						slen = strlen(str);
 
 						if (!(mallocedval = ECPGalloc(slen + sizeof("timestamp") + 1, lineno)))
+						{
+							ECPGfree(str);
 							return false;
+						}
 
 						strcpy(mallocedval, "timestamp ");
 						/* also copy trailing '\0' */
@@ -1043,7 +1114,7 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 
 					*tobeinserted_p = mallocedval;
 					*malloced_p = true;
-					free(str);
+					ECPGfree(str);
 				}
 				break;
 
@@ -1114,6 +1185,7 @@ ECPGexecute(struct statement * stmt)
 			if (desc == NULL)
 			{
 				ECPGraise(stmt->lineno, ECPG_UNKNOWN_DESCRIPTOR, ECPG_SQLSTATE_INVALID_SQL_DESCRIPTOR_NAME, var->pointer);
+				ECPGfree(copiedquery);
 				return false;
 			}
 
@@ -1145,7 +1217,10 @@ ECPGexecute(struct statement * stmt)
 							desc_inlist.ind_offset = 0;
 						}
 						if (!ECPGstore_input(stmt->lineno, stmt->force_indicator, &desc_inlist, &tobeinserted, &malloced))
+						{
+							ECPGfree(copiedquery);
 							return false;
+						}
 
 						break;
 					}
@@ -1169,8 +1244,13 @@ ECPGexecute(struct statement * stmt)
 			 * Now tobeinserted points to an area that is to be inserted
 			 * at the first %s
 			 */
-			if (!(newcopy = (char *) ECPGalloc(strlen(copiedquery) + strlen(tobeinserted) + 1, stmt->lineno)))
+			if (!(newcopy = (char *) ECPGalloc(strlen(copiedquery)
+											 + strlen(tobeinserted)
+											 + 1, stmt->lineno)))
+			{
+				ECPGfree(copiedquery);
 				return false;
+			}
 
 			strcpy(newcopy, copiedquery);
 			if ((p = next_insert(newcopy + hostvarl)) == NULL)
@@ -1179,7 +1259,11 @@ ECPGexecute(struct statement * stmt)
 				 * We have an argument but we dont have the matched up
 				 * string in the string
 				 */
-				ECPGraise(stmt->lineno, ECPG_TOO_MANY_ARGUMENTS, ECPG_SQLSTATE_USING_CLAUSE_DOES_NOT_MATCH_PARAMETERS, NULL);
+				ECPGraise(stmt->lineno, ECPG_TOO_MANY_ARGUMENTS,
+						  ECPG_SQLSTATE_USING_CLAUSE_DOES_NOT_MATCH_PARAMETERS,
+						  NULL);
+				ECPGfree(copiedquery);
+				ECPGfree(newcopy);
 				return false;
 			}
 			else
@@ -1219,7 +1303,9 @@ ECPGexecute(struct statement * stmt)
 	/* Check if there are unmatched things left. */
 	if (next_insert(copiedquery) != NULL)
 	{
-		ECPGraise(stmt->lineno, ECPG_TOO_FEW_ARGUMENTS, ECPG_SQLSTATE_USING_CLAUSE_DOES_NOT_MATCH_PARAMETERS, NULL);
+		ECPGraise(stmt->lineno, ECPG_TOO_FEW_ARGUMENTS,
+				  ECPG_SQLSTATE_USING_CLAUSE_DOES_NOT_MATCH_PARAMETERS, NULL);
+		ECPGfree(copiedquery);
 		return false;
 	}
 
@@ -1229,7 +1315,9 @@ ECPGexecute(struct statement * stmt)
 	{
 		if ((results = PQexec(stmt->connection->connection, "begin transaction")) == NULL)
 		{
-			ECPGraise(stmt->lineno, ECPG_TRANS, ECPG_SQLSTATE_TRANSACTION_RESOLUTION_UNKNOWN, NULL);
+			ECPGraise(stmt->lineno, ECPG_TRANS,
+					  ECPG_SQLSTATE_TRANSACTION_RESOLUTION_UNKNOWN, NULL);
+			ECPGfree(copiedquery);
 			return false;
 		}
 		PQclear(results);
@@ -1372,7 +1460,7 @@ ECPGexecute(struct statement * stmt)
 }
 
 bool
-ECPGdo(int lineno, int compat, int force_indicator, const char *connection_name, char *query,...)
+ECPGdo(int lineno, int compat, int force_indicator, const char *connection_name, const char *query,...)
 {
 	va_list		args;
 	struct statement *stmt;
@@ -1382,7 +1470,7 @@ ECPGdo(int lineno, int compat, int force_indicator, const char *connection_name,
 
 	/* Make sure we do NOT honor the locale for numeric input/output */
 	/* since the database wants the standard decimal point */
-	oldlocale = strdup(setlocale(LC_NUMERIC, NULL));
+	oldlocale = ECPGstrdup(setlocale(LC_NUMERIC, NULL), lineno);
 	setlocale(LC_NUMERIC, "C");
 
 	if (!ECPGinit(con, connection_name, lineno))
@@ -1398,6 +1486,7 @@ ECPGdo(int lineno, int compat, int force_indicator, const char *connection_name,
 	{
 		setlocale(LC_NUMERIC, oldlocale);
 		ECPGfree(oldlocale);
+		free_statement(stmt);
 		return (false);
 	}
 	va_end(args);
